	page	60,132
	TITLE	  -  Low Resolution Graphics Utilities    ver 1.25  -

comment *

=============================================================================

               Low Resolution Color Graphics Utilities
              -----------------------------------------

                 Copyright(C) 1983   Windmill Software Inc.


These routines are designed to link with modules compiled by the Lattice
C Compiler and called as external C procedures.

The routines supported are:

	lowres(brdcolor)	- set screen to lowres mode and set boarder to brdcolor

	clear(color)		- clear screen to color

	pset(x,y,color)		- set point x,y to color

	xpset(x,y,color)	- exclusive-or point x,y with color

	line(xstart,ystart,xend,yend,color)	- draw line

	xline(xstart,ystart,xend,yend,color)	- exclusive-or line
 
	eline(xstart,ystart,xend,yend,color)	- erase points on a line of same color 

	put(x,y,image)		- put image on screen

	xput(x,y,image)		- exclusive-or put image on screen

	xputf(x,y,image)	- fast exclusive-or image on screen (no collison info returned)

	getasc(c,cdat)		- get data which forms character c into array cdat

	kb_init()		- trap keyboard interrupt vector to jump user routine

	kb_rst()		- restore keyboard interrupt vector

	blank()			- disable video

	display()		- re-enable video

	peek(addr)		- return data at addr

	poke(data,addr)		- put data at addr

	medres(col_set)		- Sets up med-res, clears the screen & sets color palet to col_set

	mesage()		- display the mesage to turn on the color display if a mono is being used

	prochk()		- copy protection check
	 ============== removed 5/13/83 ==================

	fleft(x,y,color)	- routines used for fill
	fright(x,y,color)
	scaneq(x,y,color,xend)
	scanne(x,y,color,xend)
	fill(color)

	retscn()		- gets data from memory buffer and puts it on the display screen

	chkjoy()		- Reads new values for joystick X-Y and buttons 1-2

	rrltop()		- Rotate top of display for semi-smoth title scroll

=============================================================================

Date Changed:	November 19, 1982
	Version:	1.01

	- horizontal sync check taken out of peek and poke
	- added kb_rst

Date Changed:	November 28, 1982
	Version:	1.02

	- added blank and display commands
	- increase throughput of fast exclusive-or put (XPUTF)

Date Changed:	December 14, 1982
	Version:	1.1

	- add keyboard decode routine to implement fire and movement keys

Date Changed:	January 12, 1983
	Version:	1.2

	- added LDIR to indicate last PDIR used
	- moved screen to right for composite nonitors
	- changed end game key to Esc
	- allow a correct re-boot for Ctrl-Alt-Del
	- add title page display for med-res
	- add mesage for mono to turn on color display

Date Changed:	January 21, 1983
	Version:	1.21

	- changed keyboard initialize routine to store program DS (data segment
	  value) in location `dataseg' in PGROUP. This was done to enable the
	  keyboard interrupt routine to restore DS to the DGROUP value without
	  using any segment relative notation. This allows the program to be
	  converted to a .com file after linking.

	- changed the keyboard control-alternate-delete to do a warm boot
	  instead of a cold boot.


Date Changed:	January 30, 1983
	Version:	1.22

	- implement copy protection check


Date Changed:	February 15, 1983
	Version:	1.23

	- limit screen shift function

Date Changed:	March 23, 1983
	Version:	1.24

	- added erase line routine

	- if a point is xor'd with black, nothing is writen to screen
	  this alows a check of a location or line for colisions

Date Changed:	April 9, 1983
	Version:	1.25

	- added routines to support fill utility (fill, fleft, fright, scaneq, scanne)

	- all screen writes are now done to a memory buffer as well as the screen

	_ screen reads are done from the memory buffer to increase speed

	- the routine retscn() gets the memory buffer screen onto the display screen
	  writes for fill routine are only done to the buffer for speed

	- a global variable SNOW will disable screen sync checking when set to TRUE

	- changed all OFFSET statements to avoid previous errors

	- clear() clears memory buffer as well as screen

	- if screen=FALSE then any writes will go only to the memory buffer

	- added joystick & button support


*

; ****** Constant Declarations ******

; ===== I/O Port assignments

CRTADD	EQU	3D4H		; address port of 6845 CRT controller
CRTDAT	EQU	3D5H		; data port of 6845

MODESEL	EQU	3D8H		; mode select register for color card
COLREG	EQU	3D9H		; color select register
CRDSTAT	EQU	3DAH		; color card status register

KB_DATA	EQU	60H		; keyboard data port
KB_CNTL	EQU	61H		; keyboard control port

COLSEG	EQU	0B800H		; color card segment

; ===== Keyboard Assignments

FALSE		EQU	0
TRUE		EQU	0FFH
STOP		EQU	0			; stopped movement
LEFT		EQU	1			; left movement
RIGHT		EQU	2			; right movement

DIRR		EQU	0			; Right movement
DIRUR		EQU	1			; Up and right movement
DIRU		EQU	2			; Up
DIRUL		EQU	3			; Up and left
DIRL		EQU	4			; Left
DIRDL		EQU	5			; Down and left
DIRD		EQU	6			; Down
DIRDR		EQU	7			; Down and right		

L_ARROW_DOWN	EQU	75			; left arrow down
L_ARROW_UP	EQU	L_ARROW_DOWN OR 128     ; left arrow up
R_ARROW_DOWN	EQU	77			; right arrow down
R_ARROW_UP	EQU	R_ARROW_DOWN OR 128	; right arrow key up
U_ARROW_DOWN	EQU	72			; up arrow
U_ARROW_UP	EQU	U_ARROW_DOWN OR 128
D_ARROW_DOWN	EQU	80			; down arrow
D_ARROW_UP	EQU	D_ARROW_DOWN OR 128
UR_ARROW_DOWN	EQU	73			; up & right (9)
UR_ARROW_UP	EQU	UR_ARROW_DOWN OR 128
UL_ARROW_DOWN	EQU	71			; up & left (7)
UL_ARROW_UP	EQU	UL_ARROW_DOWN OR 128
DR_ARROW_DOWN	EQU	81			; down & right (3)
DR_ARROW_UP	EQU	DR_ARROW_DOWN OR 128
DL_ARROW_DOWN	EQU	79			; down & left (1)
DL_ARROW_UP	EQU	DL_ARROW_DOWN OR 128

F1_DOWN		EQU	59			; F1 key down
F1_UP		EQU	F1_DOWN OR 128		; F1 key up
CNTL_DOWN	EQU	29			; control key down
CNTL_UP		EQU	CNTL_DOWN OR 128	; control key up
ALT_DOWN	EQU	56			; alternate key down
ALT_UP		EQU	ALT_DOWN OR 128		; alternate key up
DEL_DOWN	EQU	83			; delete key down
DEL_UP		EQU	DEL_DOWN OR 128		; delete key up
F9_DOWN		EQU	67			; F9 down
F9_UP		EQU	F9_DOWN OR 128		; F9 up
ESC_DOWN	EQU	1			; Esc key down
SYNC_NORM	EQU	59H			; Normal sync position



; ------ Alternate Segment Definitions ------
;
; ROMSEG - rom bios segment

ROMSEG	SEGMENT AT 0F000H

	ORG	0FA6EH

CRTCHAR	DB	?

	ORG 0FFF0H

REBOOT	PROC	FAR		; Re-boot jump vector in ROM BIOS
	ASSUME CS:ROMSEG

DUMMY	DW	?

REBOOT	ENDP

ROMSEG	ENDS



; ----- This segment references the dos data area

DOSSEG	SEGMENT AT 0

	ORG	410H

EQPFLG	DB	?

DOSSEG	ENDS



; ----- This segment references to bios data area

ROMDATA		SEGMENT	AT 40H

		ORG	72H

RESET_FLAG	DW	?		; keyboard reset flag

ROMDATA		ENDS


; ------ Data Storage Area ------

DGROUP	GROUP	DATA
DATA	SEGMENT	BYTE PUBLIC 'DATA'
	ASSUME	DS:DGROUP

	PUBLIC	KEYCHR, KEYDIR, LKDIR, DEFSEG, PDIR, LDIR, FIRE, FIRES, ENDGAME, SOUND, SNOW, SCREEN, BCOUNT
	PUBLIC	JOYX, JOYY, BUTT1, BUTT2

CRTINIT	DB	71H,50H,SYNC_NORM,0AH,7CH,06H,64H,70H,2,1,20H,0,0,0
MONINIT	DB	38H,28H,2DH,0AH,7FH,06H,64H,70H,2,1,20H,0,0,0

D1		DW	?
D2		DW	?
D3		DW	?
S1		DB	?
S2		DB	?
A1		DB	?
A2		DB	?
N1		DW	?
XSTART		DW	?
YSTART		DW	?
XEND		DW	?
YEND		DW	?
COLOR		DB	?
XSTRT		DB	?
XWID		DB	?
KB_ADR		DW	?
KB_SEG		DW	?

L_KEY		DB	0
R_KEY		DB	0
U_KEY		DB	0
D_KEY		DB	0
UR_KEY		DB	0
UL_KEY		DB	0
DR_KEY		DB	0
DL_KEY		DB	0
PDIR		DB	0
LDIR		DB	0
FIRE		DB	0
FIRES		DB	0
RST1		DB	0
RST2		DB	0
RST3		DB	0
SND_UP		DB	0FFH
SOUND		DB	0FFH
ENDGAME		DB	FALSE
SYNC_POS	DB	SYNC_NORM

KEYCHR		DB	0
DEFSEG		DW	?

OLDCOL		DB	0
LOOP		DW	0

SNOW		DW	0
SCREEN		DW	0FFFFH
XCOUNT		DW	0
XREF		DW	0
BCOUNT		DW	0
KEYDIR		DB	0
LKDIR		DB	0

JOYX		DW	0
JOYY		DW	0
BUTT1		DW	0
BUTT2		DW	0


; ----- Keyboard Decode Table

KB_TABLE	LABEL	BYTE

	DB	L_ARROW_DOWN
	DW	OFFSET PGROUP:LEFT_DOWN
	DB	L_ARROW_UP
	DW	OFFSET PGROUP:LEFT_UP

	DB	R_ARROW_DOWN
	DW	OFFSET PGROUP:RIGHT_DOWN
	DB	R_ARROW_UP
	DW	OFFSET PGROUP:RIGHT_UP

	DB	U_ARROW_DOWN
	DW	OFFSET PGROUP:UP_DOWN
	DB	U_ARROW_UP
	DW	OFFSET PGROUP:UP_UP

	DB	D_ARROW_DOWN
	DW	OFFSET PGROUP:DOWN_DOWN
	DB	D_ARROW_UP
	DW	OFFSET PGROUP:DOWN_UP

	DB	UR_ARROW_DOWN
	DW	OFFSET PGROUP:UPR_DOWN
	DB	UR_ARROW_UP
	DW	OFFSET PGROUP:UPR_UP

	DB	UL_ARROW_DOWN
	DW	OFFSET PGROUP:UPL_DOWN
	DB	UL_ARROW_UP
	DW	OFFSET PGROUP:UPL_UP

	DB	DR_ARROW_DOWN
	DW	OFFSET PGROUP:DOWNR_DOWN
	DB	DR_ARROW_UP
	DW	OFFSET PGROUP:DOWNR_UP

	DB	DL_ARROW_DOWN
	DW	OFFSET PGROUP:DOWNL_DOWN
	DB	DL_ARROW_UP
	DW	OFFSET PGROUP:DOWNL_UP


	DB	F1_DOWN
	DW	OFFSET PGROUP:FIRE_DOWN
	DB	F1_UP
	DW	OFFSET PGROUP:FIRE_UP
	DB	CNTL_DOWN
	DW	OFFSET PGROUP:RESET1_DOWN
	DB	CNTL_UP
	DW	OFFSET PGROUP:RESET1_UP
	DB	ALT_DOWN
	DW	OFFSET PGROUP:RESET2_DOWN
	DB	ALT_UP
	DW	OFFSET PGROUP:RESET2_UP
	DB	DEL_DOWN
	DW	OFFSET PGROUP:RESET3_DOWN
	DB	DEL_UP
	DW	OFFSET PGROUP:RESET3_UP
	DB	F9_DOWN
	DW	OFFSET PGROUP:SOUND_DOWN
	DB	F9_UP
	DW	OFFSET PGROUP:SOUND_UP
	DB	ESC_DOWN
	DW	OFFSET PGROUP:EXIT_DOWN


NUM_CHR	EQU	($-KB_TABLE)/3

SDATA	DB	8000 DUP(0)	; Secondary screen buffer area



ERRMSG	DB 0AH,0DH
	DB	'This program requires the color graphics adapter card $'

MONMSG	DB	0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH
	DB	'                    Please turn on the monitor connected to'
	DB	0AH,0DH,0AH,0DH
	DB	'                    to your Color/Graphics display adapter. '
	DB	0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH,0DH
	DB	'$'


	
DATA	ENDS





; ------ Program Area ------

PGROUP	GROUP	PROG
PROG	SEGMENT	BYTE PUBLIC 'PROG'
	PUBLIC	LOWRES, CLEAR, PSET, XPSET, LINE, ELINE, XLINE, GETASC, PUT, XPUT, PUTF
	PUBLIC	KB_INIT, KB_RST, PEEK, POKE, MEDRES, MESAGE
	PUBLIC  FILL, FRIGHT, FLEFT, SCANEQ, SCANNE, GETSCN, RETSCN, RRLTOP, CHKJOY


; *********************************
	ASSUME	CS:PGROUP,DS:DGROUP,ES:DGROUP

CSDATA	PROC	NEAR

DATASEG	DW	?		; location to store value of data segment

CSDATA	ENDP


	


; ========== Medium Res Title Initializtion Routine ==========
;
; Calling format:
;
;       tpage(col_set)
;
; Description:
;
;       This module will intialize the color display adapter
;       to the med-res mode & clear the title page to color palette col_set.
;	The equipflag for bios is set to show a color card in use
;	A TRUE status is returned if a color card is present


MEDRES	PROC	NEAR

	PUSH	BP
	MOV	BP,SP			; point to passed parameters

	CALL	CHKDIS			; check to make sure there is a color card
	JZ	MCONT			; Display error message if no display is present
	MOV	SI,OFFSET DGROUP:ERRMSG	; Base of error mesage
PLOOP:	MOV	BX,0	
	MOV	AL,[SI]			; Get character to print
	CMP	AL,'$'			; check for end of string
	JZ	ERREXT			; exit if done
	MOV	AH,14			; use TTY write routine
	INT	10H			; do video I/O
	INC	SI
	JMP	PLOOP

ERREXT:	MOV	AX,FALSE		; Return a false value to indicate no card
	JMP	MEND

MCONT:	MOV	DX,MODESEL		; mode select port
	MOV	AL,2
	OUT	DX,AL			; disable video

	MOV	SI,OFFSET DGROUP:MONINIT	; point to 6845 initialization table
	MOV	CX,16			; set loop for 16 register
	MOV	BL,0			; start with register 0

MLR1:	MOV	DX,CRTADD		; 6845 address port
	MOV	AL,BL			; get register number
	OUT	DX,AL			; send to 6845
	MOV	DX,CRTDAT		; 6845 data port
	MOV	AL,[SI]			; get paramter
	OUT	DX,AL			; send paramter to 6845
	INC	SI			; point to next paramter
	INC	BL			; point to next register
	LOOP	MLR1			; loop until done

	MOV	DX,COLREG
	MOV	AL,0
	OUT	DX,AL	 		; and output to color select register

LCL5:	IN	AL,DX
	TEST	AL,8
	JZ	LCL5			; wait for verticle sync

	MOV	DX,MODESEL
	MOV	AL,0AH
	OUT	DX,AL			; re-enable video

	PUSH	DS
	MOV	AX,0
	MOV	DS,AX			; Set up segment
	ASSUME DS:DOSSEG

	MOV	AL,EQPFLG
	AND	AL,0CFH			; mask off old display setting
	OR	AL,10H			; Show a 40X25 color display in use
	MOV	EQPFLG,AL

	ASSUME DS:DGROUP
	
        POP	DS

	MOV	AH,0			; Code to set video mode
	MOV	AL,4			; Code for 320x200 color
	INT	10H			; Video I/O call

	MOV	AX,[BP]+4		; Get color set value
	MOV	BL,AL			; Set color palette
	MOV	BH,1			; Code to select pallette
	MOV	AH,11			; Code for set color pallette
	INT	10H
	
	MOV	AX,TRUE			; Set status for color card present

MEND:	POP	BP			; restore BP
	RET

MEDRES	ENDP




; ========== Monochrome Mesage Printing Routine ==========
;
; Calling format:
;
;       mesage();
;
; Description:
;
;       This module will display a mesage on the monochrome
;	to ask the user to turn on the color display adapter

MESAGE	PROC	NEAR

	PUSH	BP
	MOV	BP,SP			; point to passed parameters

	PUSH	DS			; Save data seg
	MOV	AX,0
	MOV	DS,AX			; Set up segment
	
	ASSUME DS:DOSSEG

	MOV	AL,EQPFLG

	ASSUME DS:DGROUP

	POP	DS			; Recover segment
	AND	AL,030H
	CMP	AL,030H
	JNZ	MEND1			; Only display message if a monochrome is attached

	MOV	CX,2000H		; Set for no cursor
	MOV	AH,1			; Set cursor command
	INT	10H			; Do video I/O

	MOV	SI,OFFSET DGROUP:MONMSG	; Base of monitor change mesage
PLOOP1:	MOV	BX,0	
	MOV	AL,[SI]			; Get character to print
	CMP	AL,'$'			; check for end of string
	JZ	MEND1			; exit if done
	MOV	AH,14			; use TTY write routine
	INT	10H			; do video I/O
	INC	SI
	JMP	PLOOP1

MEND1:	POP	BP			; restore BP
	RET

MESAGE	ENDP




; ========== Joystick Read Routine ==========
;
; Calling format:
;
;       chkjoy()
;
; Description:
;
;       This module will read new values from the joystick adapter
;		JOYX = X value
;		JOYY = Y value
;		BUTT1 - Set to TRUE when button 1 is depressed
;		BUTT2 - Set to TRUE when button 2 is depressed
;


CHKJOY	PROC	NEAR

	PUSH	BP
	MOV	BP,SP		; point to passed parameters

	MOV	DX,201H		; Game adapter port number

; read button status

	MOV	ES:BUTT1,FALSE	; Reset button status to false
	MOV	ES:BUTT2,FALSE
	IN	AL,DX		; Read buttons
	TEST	AL,10H		; Check button #1
	JNZ	NBUT1
	MOV	ES:BUTT1,TRUE	; Button 1 is depressed
NBUT1:	TEST	AL,20H		; Check button #2
	JNZ	NBUT2
	MOV	ES:BUTT2,TRUE	; Button 2 is depressed

; read joystick values

NBUT2:	MOV	BX,0		; Reset X value
	MOV	CX,0		; Reset Y value
	MOV	AH,0		; Set AH for null addition

	OUT	DX,AL		; Start timer pulse
JOY1:	IN	AL,DX		; Get new status

	RCR	AL,1		; Get X status into carry
	ADC	BL,AH	
	RCR	AL,1		; Get Y status into carry
	ADC	CL,AH
	RCR	AL,1
	INC	CH		; Check for time out
	JNZ	JOY1
	
JOYDON:	MOV	CH,0		; Reset CH after use as a timer
	MOV	ES:JOYX,BX	; Save X value
	MOV	ES:JOYY,CX	; Save Y value

	POP	BP		; restore BP
	RET


;	TEST	AL,1		; Check X status
;	JZ	NJOY1		; Skip if already timed out
;	INC	BX		; Increment X value
;	NOP			; Balance timing
;	NOP
;	NOP
;NJOY1:	TEST	AL,2		; Check Y status
;	JZ	NJOY2		; Skip if already timed out
;	INC	CX		; Increment Y value
;	NOP			; Balance timing
;	NOP
;	NOP
;NJOY2:	INC	JOYCNT
;	JZ	JOYDON		; Exit after time out
;	AND	AL,3		; Check if either X or Y is still active
;	JNZ	JOY1		; Continue checking status


CHKJOY	ENDP




; ========== Lowres Initializtion Routine ==========
;
; Calling format:
;
;       lowres(bdcolor)
;       int	bdcolor;        /*  boarder color  */
;
; Description:
;
;       This module will intialize the color display adapter
;       to the low-res mode (160 X 100 pixel by 16 color
;       resolution). The boarder color will also be set to
;       the color passed to this routine.


LOWRES	PROC	NEAR

	PUSH	BP
	MOV	BP,SP		; point to passed parameters

	CALL	CHKDIS		; check to make sure there is a color card
	JNZ	LEND		; jump if no color card

	CALL	LRES		; switch to low-res mode

LEND:	POP	BP		; restore BP
	RET

; ---------- check for color card

CHKDIS:

	PUSH	DS		; save data seg

	MOV	AX,COLSEG
	MOV	DS,AX		; set DS to color card segment

	MOV	AX,55		; set AX to a value
	MOV	BX,0		; set BX to beginning of card
	MOV	[BX],AX		; write a word to the screen
	SUB	AX,[BX]		; compare words to see if equal

	POP	DS		; restore data seg

CKEND:	RET


; ---------- switch to low-res mode

LRES:

	MOV	DX,CRDSTAT
LRES1:	IN	AL,DX
	TEST	AL,8
	JZ	LRES1
	MOV	DX,MODESEL
	MOV	AL,1
	OUT	DX,AL

	MOV	SI,OFFSET DGROUP:CRTINIT	; point to 6845 initialization table
	MOV	CX,16			; set loop for 16 register
	MOV	BL,0			; start with register 0

LR1:	MOV	DX,CRTADD		; 6845 address port
	MOV	AL,BL			; get register number
	OUT	DX,AL			; send to 6845
	MOV	DX,CRTDAT		; 6845 data port
	MOV	AL,[SI]			; get paramter
	OUT	DX,AL			; send paramter to 6845
	INC	SI			; point to next paramter
	INC	BL			; point to next register
	LOOP	LR1			; loop until done

	MOV	DX,COLREG
	MOV	AX,[BP]+4	; get passed boarder color
	OUT	DX,AL	 	; and output to color select register

	PUSH	DS

	MOV	CX,8000		; loop 8000 times

	MOV	BX,00DEH	; fill attribute with 00 and charater byte with 0DDH
	MOV	DX,CRDSTAT	; color card status register port
	MOV	AX,COLSEG
	MOV	DS,AX		; set DS to color card segment
	MOV	DI,0		; point to beginning of color card

CL2:	IN	AL,DX
	TEST	AL,8
	JZ	CL2		; wait for verticle sync

	MOV	DX,MODESEL	; mode select port
	MOV	AL,1
	OUT	DX,AL		; disable video

CL3:	MOV	[DI],BX		; write word in AX to card
	INC	DI
	INC	DI		; point to next word
	LOOP	CL3		; loop until done

	MOV	DX,CRDSTAT	; color card status port

CL5:	IN	AL,DX
	TEST	AL,8
	JZ	CL5		; wait for verticle sync

	MOV	DX,MODESEL
	MOV	AL,9
	OUT	DX,AL		; re-enable video

	POP	DS

	RET

LOWRES	ENDP



; ========== gets data from memory buffer and puts it on the display screen ==========
;
; Calling format:
;
;       retscn()
;
; Description:
;
;       This routine will take the screen image from the memory buffer
;	and put it on the display screen.  This is required as the fill routines only
;	use the memory buffer.


RETSCN	PROC	NEAR

	PUSH	BP
	MOV	BP,SP			; point to passed parameters

	PUSH	DS			; Save data seg
	MOV	AX,COLSEG
	MOV	DS,AX			; Set up segment for color card 


	MOV	DI,1			; Set start address of color card data
	MOV	SI,OFFSET DGROUP:SDATA		; Point to start of memory buffer
	MOV	CX,8000			; Number of bytes to be moved
	MOV	DX,CRDSTAT	; color card status port

RLOOP:	MOV	AH,ES:[SI]		; Get byte
	CMP	ES:SNOW,0
	JNZ	NWAIT1	
RA1:	IN	AL,DX
	TEST	AL,1
	JNZ	RA1		; wait for active display

RA2:	IN	AL,DX
	TEST	AL,1
	JZ	RA2		; wait for non active display

NWAIT1:	MOV	[DI],AH			; move byte
	INC	DI
	INC	DI
	INC	SI
	LOOP	RLOOP			; Loop untill done

	POP	DS			; Recover segment
	POP	BP			; restore BP
	RET

RETSCN	ENDP



; ========== gets data from first 8 lines of memory buffer and puts it on the display screen ==========
;
; Calling format:
;
;       rettop()
;
; Description:
;
;       This routine will take the screen image of the top 8 lines from the memory buffer
;	and put it on the display screen.  This is required as the fill routines only
;	use the memory buffer.


RRLTOP	PROC	NEAR

	PUSH	BP
	MOV	BP,SP			; point to passed parameters

	PUSH	DS			; Save data seg
	MOV	AX,COLSEG
	MOV	DS,AX			; Set up segment for color card 
	MOV	ES,AX			; Set up segment for color card 


	MOV	DI,161			; Set start address of color card data
	MOV	SI,163			; Set start address of color card data
	CLD				; Set direction to increment

	MOV	CX,560			; Number of bytes to be moved
	MOV	DX,CRDSTAT	; color card status port
RRA1:	IN	AL,DX
	TEST	AL,8
	JNZ	RRA1		; wait for active display
RRA2:	IN	AL,DX
	TEST	AL,8
	JZ	RRA2		; wait for non active display

	MOV	DX,MODESEL
	MOV	AL,1
	OUT	DX,AL		; Disable video	

	MOV	DX,CRDSTAT	; color card status port
	REP	MOVSW			; Move word


	MOV	DX,CRDSTAT	; color card status port
RRC2:	IN	AL,DX
	TEST	AL,8
	JZ	RRC2		; wait for non active display

	MOV	DX,MODESEL
	MOV	AL,9
	OUT	DX,AL		; Re-enable video	



	POP	DS			; Recover segment
	MOV	AX,DS
	MOV	ES,AX
	POP	BP			; restore BP
	RET

RRLTOP	ENDP


;
;	MOV	CX,180			; Number of bytes to be moved
;RRA1:	IN	AL,DX
;	TEST	AL,8
;	JNZ	RRA1		; wait for active display
;RRA2:	IN	AL,DX
;	TEST	AL,8
;	JZ	RRA2		; wait for non active display
;	REP	MOVSW			; Move word
;
;
;	MOV	CX,100			; Number of bytes to be moved
;RRB1:	IN	AL,DX
;	TEST	AL,1
;	JNZ	RRB1		; wait for active display
;RRB2:	IN	AL,DX
;	TEST	AL,1
;	JZ	RRB2		; wait for non active display
;	MOV	BX,[SI]
;RRB3:	IN	AL,DX
;	TEST	AL,1
;	JNZ	RRB3		; wait for active display
;RRB4:	IN	AL,DX
;	TEST	AL,1
;	JZ	RRB4		; wait for non active display
;	MOV	[DI],BX
;	INC	DI
;	INC	DI
;	INC	SI
;	INC	SI
;
;	LOOP	RRB1
;
;
;	MOV	CX,180			; Number of bytes to be moved
;RRC2:	IN	AL,DX
;	TEST	AL,8
;	JZ	RRC2		; wait for non active display
;	REP	MOVSW			; Move word
;
;	MOV	CX,100			; Number of bytes to be moved
;RRD1:	IN	AL,DX
;	TEST	AL,1
;	JNZ	RRD1		; wait for active display
;RRD2:	IN	AL,DX
;	TEST	AL,1
;	JZ	RRD2		; wait for non active display
;	MOV	BX,[SI]
;RRD3:	IN	AL,DX
;	TEST	AL,1
;	JNZ	RRD3		; wait for active display
;RRD4:	IN	AL,DX
;	TEST	AL,1
;	JZ	RRD4		; wait for non active display
;	MOV	[DI],BX
;	INC	DI
;	INC	DI
;	INC	SI
;	INC	SI
;
;	LOOP	RRD1
;


GETSCN	PROC	NEAR
;
	PUSH	BP
	MOV	BP,SP			; point to passed parameters

	PUSH	DS			; Save data seg
;	MOV	AX,COLSEG
;	MOV	DS,AX			; Set up segment for color card 
;
;
;	MOV	DI,1			; Set start address of color card data
;	MOV	SI,OFFSET DGROUP:SDATA		; Point to start of memory buffer
;	MOV	CX,8000			; Number of bytes to be moved
;
;GLOOP:	MOV	AL,[DI]			; Get byte
;	MOV	ES:[SI],AL		; move byte
;	INC	DI
;	INC	DI
;	INC	SI
;	LOOP	GLOOP			; Loop untill done
;
	POP	DS			; Recover segment
	POP	BP			; restore BP
	RET

GETSCN	ENDP


; ========== Fill all black locations on screen ==========
;
; Calling format:
;
;       fill(color)
;       int	color;		/*  color to fill screen with  */
;
; Description:
;
;       This routine will fill all black cells of the graphics screen 
;	in the area 2,10 - 157,97 to the color passed to this routine.


FILL	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	BX,[BP]+4	; get passed color
	AND 	BX,000FH	; mask off unecessary bits
	MOV	BH,BL		; copy color to BH
	MOV	CL,4
	SHL	BH,CL		; shift lower nible to upper nibble

	MOV	CX,86		; loop 86 times
	MOV	DI,OFFSET DGROUP:SDATA
	ADD	DI,881		; set pointer to first attibute byte of the 10th line

LLOOP:	MOV	ES:LOOP,0	; Reset line loop counter

NWAIT3:	MOV	AL,[DI]		; get attribute byte
	MOV	AH,AL		; save attribute
	AND	AL,0FH		; look at lower cell
	CMP	AL,8
	JZ	FSKP1		; Dont color over grey
	TEST	AL,8H		; Check if high intensity color
	JNZ	MASK1
	AND	AL,0FH		; Re-activate flags
	JNZ	FSKP1		; Do nothing if it is a color other than black
MASK1:	AND	AH,0F0H		; Keep high order bits
	OR	AH,BL		; add new color
FSKP1:	MOV	AL,AH		; save for later
	AND	AH,0F0H		; look at upper cell
	CMP	AH,80H
	JZ	FSKP2		; Dont color over grey
	TEST	AH,80H		; Check for high intensity colors
	JNZ	MASK2
	AND	AH,0F0H		; Re-activate flags
	JNZ	FSKP2		; Do nothing if it is a color other than black
MASK2:	AND	AL,0FH		; Keep low order bits
	OR	AL,BH		; add new color
FSKP2:	MOV	[DI],AL		; Save new attribute	

	INC	DI
	INC	ES:LOOP
	CMP	ES:LOOP,78	; Check if line is done
	JB	NWAIT3
	ADD	DI,2		; Correct DI fir start of next line

	LOOP	LLOOP		; loop until done

	POP	BP

	RET

FILL	ENDP





; ======================= Scan line to find the first cell not of the passed color =================
;
; Calling format:
;
;       scaneq(x,y,color,xend);
;	returns a new value for x
;
;
; Description:
;
;       This module will check points along the line for the
;	first point NOT equal to the passed color.
;	The equivelent C routine was:
;
;		while(xpset(x,y,BLACK)==old_mask && x<xend)
;			x++;
;
;	old_mask is the equivelent color colision mask for color
;


SCANEQ	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; store the x coordinate in BL
	MOV	XCOUNT,AX	; X is also needed by fill routine
	MOV	AX,[BP]+6
	MOV	BH,AL		; store the y coordinate in BH
	MOV	AX,[BP]+8
	AND	AL,0FH		; look only at lower nibble
	MOV	CH,AL		; store color in CH
	MOV	CL,CH		; store low nibble color equiv in CL
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1		; store high nibble color in CH
	MOV	AX,[BP]+10
	MOV	XREF,AX		; Used for end of line checking

	CALL	CHKPARM		; check x,y coordinates
	JZ	FOK3
	JMP	SCNDON

FOK3:	CALL	GETADR
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to memory data area
	TEST	AX,1
	JNZ	SLONIB		; Start at low nibble

SHINIB:	MOV	AL,[DI]		; Get attribute
	AND	AL,0F0H		; Look at hi-nibble only
	CMP	AL,CH		; is it equal to new color
	JNZ	SCNDON		; Exit when another color is found
	MOV	AX,ES:XREF
	CMP	ES:XCOUNT,AX	; Check end limit
	JAE	SCNDON
	INC	ES:XCOUNT	; increment counter
SLONIB:	MOV	AL,[DI]		; Get attribute
	AND	AL,0FH		; Look at low-nibble only
	CMP	AL,CL		; is it equal to new color
	JNZ	SCNDON		; Exit when another color is found
	MOV	AX,ES:XREF
	CMP	ES:XCOUNT,AX	; Check end limit
	JAE	SCNDON
	INC	ES:XCOUNT	; increment counter

	INC	DI		; point to next byte
	JMP	SHINIB

SCNDON:	MOV	AX,ES:XCOUNT	; Return new X value

	POP	BP

	RET                

SCANEQ	ENDP




; ======================= Scan line to find the first cell that is of the passed color =================
;
; Calling format:
;
;       scanne(x,y,color,xend);
;	returns a new value for x
;
;
; Description:
;
;       This module will check points along the line for the
;	first point EQUAL to the passed color.
;	The equivelent C routine was:
;
;		while(xpset(x,y,BLACK)!=old_mask && x<xend)
;			x++;
;
;	old_mask is the equivelent color colision mask for color
;


SCANNE	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; store the x coordinate in BL
	MOV	XCOUNT,AX	; X is also needed by fill routine
	MOV	AX,[BP]+6
	MOV	BH,AL		; store the y coordinate in BH
	MOV	AX,[BP]+8
	AND	AL,0FH		; look only at lower nibble
	MOV	CH,AL		; store color in CH
	MOV	CL,CH		; store low nibble color equiv in CL
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1		; store high nibble color in CH
	MOV	AX,[BP]+10
	MOV	XREF,AX		; Used for end of line checking

	CALL	CHKPARM		; check x,y coordinates
	JZ	FOK4
	JMP	SCNDN

FOK4:	CALL	GETADR
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to memory data area
	TEST	AX,1
	JNZ	SLONB		; Start at low nibble

SHINB:	MOV	AL,[DI]		; Get attribute
	AND	AL,0F0H		; Look at hi-nibble only
	CMP	AL,CH		; is it equal to new color
	JZ	SCNDN		; Exit when same color is found
	MOV	AX,ES:XREF
	CMP	ES:XCOUNT,AX	; Check end limit
	JAE	SCNDN
	INC	ES:XCOUNT	; increment counter
SLONB:	MOV	AL,[DI]		; Get attribute
	AND	AL,0FH		; Look at low-nibble only
	CMP	AL,CL		; is it equal to new color
	JZ	SCNDN		; Exit when same color is found
	MOV	AX,ES:XREF
	CMP	ES:XCOUNT,AX	; Check end limit
	JAE	SCNDN
	INC	ES:XCOUNT	; increment counter

	INC	DI		; point to next byte
	JMP	SHINB

SCNDN:	MOV	AX,ES:XCOUNT	; Return new X value

	POP	BP

	RET                

SCANNE	ENDP





; ======================= Fill line to right - used for polyfill =================
;
; Calling format:
;
;       fright(x,y,color);
;	returns a new value for x
;
;
; Description:
;
;       This module will check points along the line to the right of the start point
;	If a point is the boundary color (grey or dark white) or if the point is the new color
;	or if the right limit is reached then the routine exits
;	If these conditions are not met then the point is colored to the new color
; 	and the next point is checked.
;
;	The equivelent C routine was:
;
;		while(get(x,y)!=boundary_color && get(x,y)!=new_color && x<R_LIM) {
;			set(x,y);
;			bcount++; x++;
;		}
;
;	get(x,y) returns the color of cell x,y
;	set(x,y) sets cell x,y to the new fill color
;	bcount is a counter for the number of dots filled
;


FRIGHT	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; store the x coordinate in BL
	MOV	XCOUNT,AX	; X is also needed by fill routine
	MOV	AX,[BP]+6
	MOV	BH,AL		; store the y coordinate in BH
	MOV	AX,[BP]+8
	AND	AL,0FH		; look only at lower nibble
	MOV	CH,AL		; store color in CH
	MOV	CL,CH		; store low nibble color equiv in CL
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1		; store high nibble color in CH

	CALL	CHKPARM		; check x,y coordinates
	JZ	FOK1
	JMP	FRDONE

FOK1:	CALL	GETADR
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to memory data area
	TEST	AX,1
	JNZ	FLONIB		; Start at low nibble

FHINIB:	MOV	AL,[DI]		; Get attribute
	MOV	AH,AL		; save attribute
	AND	AL,0F0H		; Look at hi-nibble only
	CMP	AL,70H		; is it dark white
	JZ	FRDONE		; Check for exit conditions
	CMP	AL,80H		; is it grey
	JZ	FRDONE		; Check for exit conditions
	CMP	AL,CH		; is it already new color
	JZ	FRDONE		; Check for exit conditions
	CMP	ES:XCOUNT,160	; Check right limit
	JAE	FRDONE
	INC	ES:XCOUNT	; increment x position counter
	INC	ES:BCOUNT	; increment dot filled counter
	AND	AH,0FH		; Save lower nibble
	OR	AH,CH		; add new color
	MOV	[DI],AH		; Save new attribute
FLONIB:	MOV	AL,[DI]		; Get attribute
	MOV	AH,AL		; save attribute
	AND	AL,0FH		; Look at low-nibble only
	CMP	AL,7H		; is it dark white
	JZ	FRDONE		; Check for exit conditions
	CMP	AL,8H		; is it grey
	JZ	FRDONE		; Check for exit conditions
	CMP	AL,CL		; is it already new color
	JZ	FRDONE		; Check for exit conditions
	CMP	ES:XCOUNT,160	; Check right limit
	JAE	FRDONE
	INC	ES:XCOUNT	; increment x position counter
	INC	ES:BCOUNT	; increment dot filled counter
	AND	AH,0F0H		; Save upper nibble
	OR	AH,CL		; add new color
	MOV	[DI],AH		; Save new attribute

	INC	DI		; point to next byte
	JMP	FHINIB

FRDONE:	MOV	AX,ES:XCOUNT	; Return new X value

	POP	BP

	RET                

FRIGHT	ENDP


; ======================= Fill line to left - used for polyfill =================
;
; Calling format:
;
;       fleft(x,y,color);
;	returns a new value for x
;
;
; Description:
;
;       This module will check points along the line to the left of the start point
;	If a point is the boundary color (grey or dark white) or if the point is the new color
;	or if the left limit is reached then the routine exits
;	If these conditions are not met then the point is colored to the new color
; 	and the next point is checked.
;
;	The equivelent C routine was:
;
;		while(get(x,y)!=boundary_color && get(x,y)!=new_color && x>L_LIM) {
;			set(x,y);
;			x++;
;		}
;
;	get(x,y) returns the color of cell x,y
;	set(x,y) sets cell x,y to the new fill color
;


FLEFT	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; store the x coordinate in BL
	MOV	XCOUNT,AX	; X is also needed by fill routine
	MOV	AX,[BP]+6
	MOV	BH,AL		; store the y coordinate in BH
	MOV	AX,[BP]+8
	AND	AL,0FH		; look only at lower nibble
	MOV	CH,AL		; store color in CH
	MOV	CL,CH		; store low nibble color equiv in CL
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1
	ROL	CH,1		; store high nibble color in CH

	CALL	CHKPARM		; check x,y coordinates
	JZ	FOK2
	JMP	FLDONE

FOK2:	CALL	GETADR
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to memory data area
	TEST	AX,1
	JZ	FHINB		; Start at low nibble

FLONB:	MOV	AL,[DI]		; Get attribute
	MOV	AH,AL		; save attribute
	AND	AL,0FH		; Look at low-nibble only
	CMP	AL,7H		; is it dark white
	JZ	FLDONE		; Check for exit conditions
	CMP	AL,8H		; is it grey
	JZ	FLDONE		; Check for exit conditions
	CMP	AL,CL		; is it already new color
	JZ	FLDONE		; Check for exit conditions
	CMP	ES:XCOUNT,0	; Check left limit
	JB	FLDONE
	INC	ES:BCOUNT	; increment dot filled counter
	DEC	ES:XCOUNT	; decrement x position counter
	AND	AH,0F0H		; Save upper nibble
	OR	AH,CL		; add new color
	MOV	[DI],AH		; Save new attribute
FHINB:	MOV	AL,[DI]		; Get attribute
	MOV	AH,AL		; save attribute
	AND	AL,0F0H		; Look at hi-nibble only
	CMP	AL,70H		; is it dark white
	JZ	FLDONE		; Check for exit conditions
	CMP	AL,80H		; is it grey
	JZ	FLDONE		; Check for exit conditions
	CMP	AL,CH		; is it already new color
	JZ	FLDONE		; Check for exit conditions
	CMP	ES:XCOUNT,0	; Check left limit
	JB	FLDONE
	INC	ES:BCOUNT	; increment dot filled counter
	DEC	ES:XCOUNT	; decrement x position counter
	AND	AH,0FH		; Save lower nibble
	OR	AH,CH		; add new color
	MOV	[DI],AH		; Save new attribute

	DEC	DI		; point to next byte
	JMP	FLONB

FLDONE:	MOV	AX,ES:XCOUNT	; Return new X value
	POP	BP
	RET                

FLEFT	ENDP



; ========== Clear Screen ==========
;
; Calling format:
;
;       clear(color)
;       int	color;		/*  color to clear screen with  */
;
; Description:
;
;       This routine will clear the graphics screen to
;       the color passed to this routine.


CLEAR	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	BX,[BP]+4	; get passed color
	AND 	BX,000FH	; mask off unecessary bits
	MOV	BH,BL		; copy color to BH
	MOV	CL,4
	SHL	BH,CL		; shift lower nible to upper nibble
	OR	BL,BH		; BL now contains color in both nibbles

	MOV	DX,CRDSTAT	; card status port
	MOV	CX,8000		; loop 8000 times
	MOV	DI,1		; set pointer to first attibute byte
	MOV	SI,OFFSET DGROUP:SDATA	; Point to memory screen buffer

	PUSH	DS		; save DS

	MOV	AX,COLSEG
	MOV	DS,AX		; set DS to color card segment

CLR2:	IN	AL,DX
	TEST	AL,8
	JZ	CLR2		; wait for verticle sync

	MOV	DX,MODESEL	; mode select register
	MOV	AL,1
 	OUT	DX,AL		; disable video

CLR3:	MOV	[DI],BL		; write byte to display attribute byte
	MOV	ES:[SI],BL	; write to memory buffer
	INC	SI		; Point to next byte
	INC	DI
	INC	DI		; point to next attibute byte
	LOOP	CLR3		; loop until done

	MOV	DX,CRDSTAT	; color card status

CLR5:	IN	AL,DX
	TEST	AL,8
	JZ	CLR5		; wait for verticle sync

	MOV	DX,MODESEL
	MOV	AL,9
	OUT	DX,AL		; re-enable video
                                                 
	POP	DS		; restore data segment
	POP	BP

	RET

CLEAR	ENDP



; ========== Set Point Routine ==========
;
; Description:
;
;       This routine will set point X,Y to the color
;       specified and return the previous color of
;       pixel as a bit set in AX.
;
; Command format:
;
;       pset(x,y,color)
;       int	x, y, color;
;
; On Exit:
;
;       AX contains a bit set representing the old color
;       of the pixel just written.
;
;          Bit  0 = collision with color 0
;               1 =     "       "    "   1
;              15 =     "       "    "   15
;               8 = coordinate was out of bounds
;
;	note:	a collision with color 8 (grey) cannot be detected therefor
;		it is not recommended that anything with the color 8 be
;		plotted.


PSET	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; store the x coordinate in BL
	MOV	AX,[BP]+6
	MOV	BH,AL		; store the y coordinate in BH
	MOV	AX,[BP]+8
	AND	AL,0FH		; look only at lower nibble
	MOV	CH,AL		; store color in CH
	MOV	BP,0		; set BP to show normal write
	MOV	SI,0		; clear collision detection flags

	PUSH	DS		; save data seg

	MOV	AX,COLSEG
	MOV	DS,AX		; set DS to color card segment

	CALL	WRTDOT

	MOV	AX,SI		; move collision flags into AX

	POP	DS		; restore data seg
	POP	BP

	RET                

PSET	ENDP



; ========== Exclusive-Or Point ==========
;
; Description:
;
;       This routine will exclusive-or point x,y with
;       the color that was passed and returns the previous
;       color of the pixel as a bit set in the returned
;       parameter.
;
; Command Format:
;
;       xpset(x,y,color)
;       int	x, y, color;
;
; On Exit:
;
;       AX has a bit set representing the previous color
;       of the pixel at x,y.


XPSET	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; move the x coordinate into BL
	MOV	AX,[BP]+6
	MOV	BH,AL		; move the y coordinate into BH
	MOV	AX,[BP]+8
	AND	AL,0FH		; look at lower nibble
	MOV	CH,AL		; move the color into CH
	MOV	BP,1		; set BP to show exclusive-or write
	MOV	SI,0		; clear collision detection flags

	PUSH	DS		; save data seg

	MOV	AX,COLSEG
	MOV	DS,AX		; set DS to color card segment

	CALL	WRTDOT

	MOV	AX,SI		; put collision value in AX to be returned

	POP	DS		; restore data seg
	POP	BP

	RET

XPSET	ENDP



; ========== Write Dot ==========
;
; On Entry:
;
;       BL = x coordinate
;       BH = y coordinate
;       CH = color to write
;       BP = 0 for normal write,
;            1 for exclusive-or write,
;            2 for fast exclusive-or write (no collision detection)
;       DS = segment of color card (0B800H)
;
; On Exit: (except for fast exclusive-or write)
;
;       SI has a bit set representing the old color of the pixel
;
;          Bit  0 = collision with color 0
;               1 =     "       "    "   1
;              15 =     "       "    "   15
;               8 = coordinate was out of bounds
;
;	note:	a collision with color 8 (grey) cannot be detected therefor
;		it is not recommended that anything with the color 8 be
;		plotted.


WRTDOT	PROC	NEAR

	CALL	CHKPARM		; check X,Y coordinates
	JZ	DOTCK0		; jump if coordinates are valid
	RET			; return if coordinates are not valid

DOTCK0:	CMP	BP,0  
	JNZ	DOTCK1
	JMP	DOTOK0		; if BP=0 then jump to normal write
DOTCK1:	CMP	BP,1  
	JNZ	DOTCK2
	JMP	DOTOK0		; if BP=1 then jump to normal write
DOTCK2:	CMP	BP,2  
	JNZ	DOTCK3
	JMP	FASTWRT		; if BP=2 then jump to fast write routine
DOTCK3:	CMP	BP,3  
	JNZ	DOTCK4
	JMP	ERADOT 		; must be an erase dot 
DOTCK4:	JMP	DOTOK0		; if BP=? then jump to normal write



DOTOK0:	CALL	GETADR		; get address of attribute
	TEST	AX,1
	JZ	HINIB		; jump if bit not set to do hi nibble


; ---------- change lower nibble

LOWNIB:	CALL	RATTR		; get attribute byte
	PUSH	AX		; save attribute
	AND	AL,0FH		; get lower nibble
	MOV	DX,1		; set DX for collision check
	MOV	CL,AL		; move color to CL
	SHL	DX,CL		; shift DX to move bit to indicate old color
	OR	SI,DX		; set collision detection into SI

	POP	AX		; get old attribute
	CMP	BP,1
	JE	XWRT		; jump if exclusive-or write

NLWRT:	AND	AL,0F0H		; clear lower nibble
	OR	AL,CH		; put new color in lower nibble
	CALL	WATTR		; write new attribute
	RET

XWRT:	XOR	AL,CH		; exclusive-or new color with old
	CMP	CH,0		; if color is 0 then do NOT re-write dot
	JZ	SKP1
	CALL	WATTR		; write new attribute
SKP1:	RET

; ---------- change upper nibble

HINIB:	CALL	RATTR		; get attribute
	PUSH	AX		; save attribute
	AND	AL,0F0H		; clear lower nibble
	MOV	CL,4
	SHR	AL,CL		; shift high nibble to lower nibble
	MOV	DX,1		; set DX for collision check
	MOV	CL,AL
	SHL	DX,CL		; shift bit to indicate old color
	OR	SI,DX		; save collision in SI
	MOV	CL,4
	SHL	CH,CL		; move new color to proper nibble
	POP	AX		; get old attribute
	CMP	BP,1
	JE	XWRT		; jump if exclusive-or write

NUWRT:	AND	AL,0FH		; clear upper nibble
	OR	AL,CH		; set new color
      	CALL	WATTR		; write new attribute
      	RET


; ---------- Fast Write Routine

FASTWRT:

	CALL	GETADR		; get address of attribute

	MOV	DX,CRDSTAT

	CMP	ES:SNOW,0
	JNZ	NWAIT4
	CMP	ES:SCREEN,0	; Set zero flag if no screen write is needed
	JZ	NSCRN4

FW1:	IN	AL,DX
	TEST	AL,1
	JNZ	FW1		; wait for active display

FW2:	IN	AL,DX
	TEST	AL,1
	JZ	FW2		; wait for non-active display

NWAIT4:	MOV	[DI],CH		; exclusive-or dot	
NSCRN4:	PUSH	DI
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to memory buffer
	MOV	ES:[DI],CH	; Write to memory buffer
	POP	DI

; *********************************

	RET



; --------------- erase dot routine --------------


ERADOT:	CALL	GETADR		; get address of attribute
	TEST	AX,1
	JZ	HINBE		; jump if bit not set to do hi nibble


; ---------- change lower nibble

LOWNBE:	CALL	RATTR		; get attribute byte
	PUSH	AX		; save attribute
	AND	AL,0FH		; get lower nibble
	MOV	ES:OLDCOL,AL	; save old screen color
	MOV	DX,1		; set DX for collision check
	MOV	CL,AL		; move color to CL
	SHL	DX,CL		; shift DX to move bit to indicate old color
	OR	SI,DX		; set collision detection into SI

	POP	AX		; get old attribute
	AND	AL,0F0H		; clear lower nibble
	CMP	ES:OLDCOL,CH	; check if colors are the same
	JNZ	NWRT1		; exit if colors are diferent
	CALL	WATTR		; write new attribute
NWRT1:	RET


; ---------- change upper nibble

HINBE:	CALL	RATTR		; get attribute
	PUSH	AX		; save attribute
	AND	AL,0F0H		; clear lower nibble
	MOV	ES:OLDCOL,AL	; save old color as high nib
	MOV	CL,4
	SHR	AL,CL		; shift high nibble to lower nibble
	MOV	DX,1		; set DX for collision check
	MOV	CL,AL
	SHL	DX,CL		; shift bit to indicate old color
	OR	SI,DX		; save collision in SI
	MOV	CL,4
	SHL	CH,CL		; move new color to proper nibble
	POP	AX		; get old attribute
	AND	AL,0FH		; clear upper nibble

	CMP	ES:OLDCOL,CH	; check if colors are the same
	JNZ	NWRT2		; exit if colors are diferent

 	CALL	WATTR		; write new attribute
NWRT2:	RET


WRTDOT	ENDP



; ---------- Parameter Check Routine
;
; This routine will check the X and Y coordinates
; and make sure thay are in range. If they are
; the SI is set to 0 and the zero flag is set else
; the MSB of SI is set and the zero flag is reset.


CHKPARM	PROC	NEAR

	CMP	BL,160
	JB	YCHK		; valid if x corordinate is under 160
	JMP	PBAD

YCHK:	CMP	BH,100
	JB	POK		; valid if y coordinate is ynder 100

PBAD:	OR	SI,10000000B	; set bit 8 to show out of bounds
	RET

POK:	XOR	AX,AX		; set zero flag
	RET

CHKPARM	ENDP



; ---------- Get Attribute Byte
;
; This routine set DI to the address of the attribute.


GETADR	PROC	NEAR

	MOV	AL,160
	MUL	BH		; AX=160*Y
	MOV	BH,0		; clear Y value - BX now contains the X coordinate
	ADD	AX,BX		; add X coordinate
	MOV	DI,AX		; move address to DI
	OR	DI,1		; set LSB - attribute byte always on odd address

	RET

GETADR	ENDP


; ---------- Read Attribute Byte
;
; This routine will read the attribute byte pointed
; to by DI into AL


RATTR	PROC	NEAR

	PUSH	DI
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to point into memory buffer
	MOV	AL,ES:[DI]	; get attribute byte
	POP	DI
 
	RET

RATTR	ENDP



; ---------- Write Attribute Byte
;
; This routine will write the byte in AL to the
; address pointed to by DI.


WATTR	PROC	NEAR

	MOV	AH,AL		; save attribute
	MOV	DX,CRDSTAT	; color card status port

	CMP	ES:SNOW,0
	JNZ	NWAIT2
	CMP	ES:SCREEN,0	; Set zero flag if no screen write is needed
	JZ	NSCRN
WA1:	IN	AL,DX
	TEST	AL,1
	JNZ	WA1		; wait for active display

WA2:	IN	AL,DX
	TEST	AL,1
	JZ	WA2		; wait for non active display

NWAIT2:	MOV	[DI],AH		; write new attribute byte
NSCRN:	PUSH	DI
	SHR	DI,1		; Divide DI by 2 and truncate
	ADD	DI,OFFSET DGROUP:SDATA	; Add offset to point into memory buffer
	MOV	ES:[DI],AH	; Store data in memory buffer
	POP	DI

	RET

WATTR	ENDP



; ========== Draw Line Routine ==========
;
; Description:
;
;       This routine will draw at line from xstart,ystart
;       to xend,yend in color.
;
; Calling Format:
;
;       line(xstart,ystart,xend,yend,color)
;       int	xstart, ystart, xend, yend, color;
;
; On Exit:
;
;       AX contains bits set repesenting colors that the line had collided
;       with.
;
;          Bit  0 = collision with color 0
;               1 =     "       "    "   1
;              15 =     "       "    "   15
;               8 = coordinate was out of bounds
;
;	note:	a collision with color 8 (grey) cannot be detected therefor
;		it is not recommended that anything with the color 8 be
;		plotted.


LINE	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	XSTART,AX	; save starting x coordinate
	MOV	AX,[BP]+6
	MOV	YSTART,AX	; save starting y coordinate
	MOV	AX,[BP]+8
	MOV	XEND,AX		; save ending x coordinate
	MOV	AX,[BP]+10
	MOV	YEND,AX		; save ending y coordinate
	MOV	AX,[BP]+12
	AND	AX,0FH
	MOV	COLOR,AL	; save color
	MOV	BP,0		; set BP to show normal line draw
	MOV	SI,0		; clear collision flags

	CALL	DLINE		; call line drawing routine

	MOV	AX,SI		; move collision detection to AX

	POP	BP
	RET

LINE	ENDP




; ========== Erase Line Routine ==========
;
; Description:
;
;       This routine will erase a line from xstart,ystart
;       to xend,yend if the color of the screen dot is color.
;
; Calling Format:
;
;       eline(xstart,ystart,xend,yend,color)
;       int	xstart, ystart, xend, yend, color;
;
; On Exit:
;
;       AX contains bits set repesenting colors that the line had collided
;       with.
;
;          Bit  0 = collision with color 0
;               1 =     "       "    "   1
;              15 =     "       "    "   15
;               8 = coordinate was out of bounds
;
;	note:	a collision with color 8 (grey) cannot be detected therefor
;		it is not recommended that anything with the color 8 be
;		plotted.


ELINE	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	XSTART,AX	; save starting x coordinate
	MOV	AX,[BP]+6
	MOV	YSTART,AX	; save starting y coordinate
	MOV	AX,[BP]+8
	MOV	XEND,AX		; save ending x coordinate
	MOV	AX,[BP]+10
	MOV	YEND,AX		; save ending y coordinate
	MOV	AX,[BP]+12
	AND	AX,0FH
	MOV	COLOR,AL	; save color
	MOV	BP,3		; set BP to show erase line 
	MOV	SI,0		; clear collision flags

	CALL	DLINE		; call line drawing routine

	MOV	AX,SI		; move collision detection to AX

	POP	BP
	RET

ELINE	ENDP




; ========== Draw Line (Exclusive-Or) Routine ==========
;
; Description:
;
;       This routine will draw at line from xstart,ystart
;       to xend,yend in color.
;
; Calling Format:
;
;       xline(xstart,ystart,xend,yend,color)
;       int	xstart, ystart, xend, yend, color;
;
; On Exit:
;
;       AX contains bits set repesenting colors that the line had collided
;       with.


XLINE	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	XSTART,AX	; save starting x coordinate
	MOV	AX,[BP]+6
	MOV	YSTART,AX	; save starting y coordinate
	MOV	AX,[BP]+8
	MOV	XEND,AX		; save ending x coordinate
	MOV	AX,[BP]+10
	MOV	YEND,AX		; save ending y coordinate
	MOV	AX,[BP]+12
	AND	AX,0FH
	MOV	COLOR,AL	; save color
	MOV	BP,1		; set BP to show exclusive-or line draw
	MOV	SI,0		; clear collision flags

	CALL	DLINE		; call line drawing routine

	MOV	AX,SI		; move collision detection to AX

	POP	BP
	RET

XLINE	ENDP




; ========== Line Draw ==========
;

DLINE	PROC	NEAR

	MOV	AX,XEND
	SUB	AX,XSTART
	MOV	D1,AX		; D1=XEND-XSTART

	MOV	AX,YEND
	SUB	AX,YSTART
	MOV	D2,AX		; D2=YEND-YSTART

	MOV	S1,0		; S1=0
	MOV	S2,1		; S2=1
	MOV	A1,1		; A1=1
	MOV	A2,0		; A2=0

L1:	CMP	D1,0
	JGE	L2		; jump if D1 >= 0
	MOV	A1,-1		; A1 = -1
	NEG	D1		; D1 = -D1

L2:	CMP	D2,0
	JGE	L3		; jump if D2 >= 0
	NEG	D2		; D2 = -D2
	MOV	S2,-1		; S2 = -1

L3:	MOV	AX,D2
	CMP	D1,AX
	JGE	L4		; jump if D1 >= D2
	PUSH	D1
	MOV	AX,D2
	MOV	D1,AX
	POP	D2		; swap D1,D2
	MOV	AL,A1
	MOV	S1,AL		; S1 = A1
	MOV	A1,0		; A1 = 0
	MOV	AL,S2
	MOV	A2,AL		; A2 = S2
	MOV	S2,0		; S2 = 0

L4:	MOV	AX,D1
	SHR	AX,1
	MOV	D3,AX		; D3 = D1/2
	MOV	N1,1		; N1 = 1

	MOV	AX,XSTART
	MOV	BL,AL		; put xstart in BL
	MOV	AX,YSTART
	MOV	BH,AL		; put ystart in BH

	CALL	PLOT		; plot xstart,ystart

LINLP:	MOV	AX,D1
	CMP	N1,AX
	JLE	LINLP1		; jump if N1 > D1
	RET

LINLP1:	ADD	BL,A1		; XSTART = XSTART + A1
	ADD	BH,A2		; YSTART = YSTART + A2
	MOV	AX,D2
	ADD	D3,AX		; D3 = D3 + D2
	INC	N1		; N1 = N1 + 1

	MOV	AX,D1
	CMP	D3,AX
	JLE	LINLP2		; jump D3 <= D1

	MOV	AX,D1
	SUB	D3,AX		; D3 = D3 - D1
	ADD	BL,S1		; XSTART = XSTART + S1
	ADD	BH,S2		; YSTART = YSTART + S2

LINLP2:	CALL	PLOT
	JMP	LINLP


; ---------- plot xstart,ystart

PLOT:

	MOV	CH,COLOR	; CH = COLOR

	PUSH	BX		; save coordinates
	PUSH	DS
	MOV	AX,COLSEG
	MOV	DS,AX		; set DS to color segment

	CALL	WRTDOT		; plot dot

	POP	DS		; restore DS
	POP	BX
	RET

DLINE	ENDP




; ========== Get ASCII Data Routine ==========
;
; Command Format:
;
;       getasc(c,cdat)
;       char    c, *cdat
;
; Description:
;
;       This routine will pass tha data which compromises
;       the the ASCII charater c into array cdat. The data
;       is taken from the rom BIOS character table.


GETASC	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	SI,[BP]+4		; get c into AX
	MOV	CL,3
	SHL	SI,CL			; shift SI left to multiply by 8
	ADD	SI,OFFSET CRTCHAR	; SI now contains the address of the character data

	MOV	DI,[BP]+6		; put address of cdata onto DI
	MOV	CX,8			; copy 8 bytes into cdata

	PUSH	DS
	MOV	AX,ROMSEG
	MOV	DS,AX			; set DS to rom segment

	REP	MOVSB			; do block move

	POP	DS
	POP	BP
	RET

GETASC	ENDP




; ========== Put Routine ==========
;
; Command Format:
;
;       put(x,y,image)
;       int     x, y;
;       char    *image;
;
; Description:
;
;       This routine will place the image at the address passed
;       in *image on the screen starting at location x,y. Collision
;       information is returned.
;
;	The image is stored in the following format:
;
;	image[] = {	xwid,ywid,
;			e1,e2,e3,
;			e4,e5,e6,
;			e7,e8,e9	};
;
;	where	xwid = x width of image
;		ywid = y width of image
;
;	note:	The way the example array is formatted represents an image
;		of 3 by 3.
;
;		e?   = image element which can have the color values
;		       0 through 15 or 88.
;
;		       If the element has a color value of 0 through 15,
;		       that color will be plotted at that point. If the
;		       element has a color vaue of 88, that point will
;		       be ignored and skiped.
;
; On Exit:
;
;       AX contains bits set repesenting colors that the image had collided
;       with.



PUT	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; move x coordinate in BL
	MOV	XSTRT,AL	; store x starting location in XSTRT
	MOV	AX,[BP]+6
	MOV	BH,AL		; move y coordinate in BH
	MOV	DI,[BP]+8	; move pointer to image array in DI
	MOV	SI,0		; clear collision counter
	MOV	BP,0		; normal write

	CALL	PUTR

	MOV	AX,SI		; put collision info in AX
	POP	BP
        RET

PUT	ENDP



; ========== Exclusive-Or Put Routine ==========
;
; Command Format:
;
;       xput(x,y,image)
;       int     x, y;
;       char    *image
;
; Description:
;
;       This routine will exclusive-or the image at the address passed
;       in *image on the screen starting at location x,y. Collision
;       information is returned.


XPUT	PROC	NEAR

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; move x coordinate in BL
	MOV	XSTRT,AL	; store x starting location in XSTRT
	MOV	AX,[BP]+6
	MOV	BH,AL		; move y coordinate in BH
	MOV	DI,[BP]+8	; move pointer to image array in DI
	MOV	SI,0		; clear collision counter
	MOV	BP,1		; exclusive-or write

	CALL	PUTR

	MOV	AX,SI		; put collision info in AX
	POP	BP
        RET

XPUT	ENDP





; ========== Fast Exclusive-Or Put Routine ==========
;
; Command Format:
;
;       xputf(x,y,image)
;       int     x, y;
;       char    *image
;
; Description:
;
;       This routine will exclusive-or the image at the address passed
;       in *image on the screen starting at location x,y. Collision
;       information is NOT returned. The image data in the array *image
;	must be in the following format:
;
;	image[] = {	xbwid, ywid,
;			0xe1e2, 0xe3e4,
;			0xe5e6, 0xe7e8	};
;
;	Since in this format one byte contains color information for two
;	pixels, xbwid is the number of bytes per line (or number of pixels / 2).
;	ywid is the verticle height of the image. If an image byte contains the
;	value 0x88, the pixels represented by this byte will be skiped and not
;	plotted on the screen.


PUTF	PROC	NEAR			; *********************************

	PUSH	BP
	MOV	BP,SP

	MOV	AX,[BP]+4
	MOV	BL,AL		; move x coordinate in BL
	MOV	XSTRT,AL	; store x starting location in XSTRT
	MOV	AX,[BP]+6
	MOV	BH,AL		; move y coordinate in BH
	MOV	DI,[BP]+8	; move pointer to image array in DI
	MOV	SI,0		; clear collision counter
	MOV	BP,2		; fast exclusive-or write

	CALL	PUTR

	MOV	AX,SI		; put collision info in AX
	POP	BP
        RET

PUTF	ENDP			; *********************************


; ---------- this routine will put the image pointed to by DI at BL,BH

PUTR	PROC	NEAR

	MOV	CL,[DI]		; put x width of image in CL
	MOV	XWID,CL		; store x width in XWID
	INC	DI
	MOV	CH,[DI]		; put y height of image in CL
	INC	DI		; DI now points to first image element

PULOOP:	CALL	POINT		; plot point
	INC	DI		; point to next image element
	INC	BL		; point to next x location

	CMP	BP,2		; check if fast exclusive-or write
	JNE	PUL1		; jump if not
	INC	BL		; increment x coordinate again if it is

PUL1:	DEC	CL
	JNZ	PULOOP		; loop until horizontal row is done
	INC	BH		; point to next horizontal row
	DEC	CH
	JZ	PUEND		; jump if finished all rows
	MOV	BL,XSTRT	; put x starting location back in BL
	MOV	CL,XWID		; put x width back in CL
	JMP	PULOOP		; repeat loop

PUEND:	RET

; ---------- this routine will plot a point at BL,BH

POINT:	PUSH	CX
	MOV	CH,[DI]		; move color into CH
	CMP	CH,88
	JE	POEND		; do not write if color is 88
	CMP	CH,88H
	JE	POEND		; do not write if color is 88H
	PUSH	BX
	PUSH	DI
	PUSH	DS
	MOV	AX,COLSEG
	MOV	DS,AX		; set new data seg

	CALL	WRTDOT		; write dot

	POP	DS
	POP	DI
	POP	BX
POEND:	POP	CX
	RET


PUTR	ENDP



; <<<<<<<<<< Keyboard Interrupt Initialization >>>>>>>>>>
;
; Command Format:
;
;       keyinit();
;
; Description:
;
;       This routine will change the keyboard interrupt vector to point to
;       a machine language routine located in this graphics library which
;       contains its own keyboard handeller.


KB_INIT	PROC	NEAR

	MOV	AX,DS
	MOV	CS:DATASEG,AX		; store DS value in location DATASEG

	PUSH	DS

	MOV	AX,0
	MOV	DS,AX			; set DS to dos seg
	MOV	DI,24H			; location of keybrd interrupt
	MOV	AX,[DI]			; get old interrupt address
	MOV	BX,[DI]+2		; get old interrupt segment
	MOV	CX,OFFSET PGROUP:KEYINT	; get address of routine
	MOV	DX,CS			; get code segment

	CLI				; disable interrupts

	MOV	[DI],CX			; store address
	MOV	[DI+2],DX		; store code seg
                                        
	STI				; enable interrupts

	POP	DS

	MOV	KB_ADR,AX		; store interrupt address
	MOV	KB_SEG,BX		; store interrupt segment

	RET

KB_INIT	ENDP



; <<<<<<<<<< Keyboard Restore Routine >>>>>>>>>>
;
; Calling Format:
;
;       kb_restore();
;
; Description:
;
;       This routine will restore the original keyboard interrupt address.


KB_RST	PROC	NEAR

	MOV	AX,KB_ADR		; get old interrupt address
	MOV	BX,KB_SEG		; get old interrupt segment
	MOV	DI,24H			; location of keyboard interrupt

	PUSH	DS			; save data segment
	MOV	CX,0
	MOV	DS,CX			; set DS to DOS segment

	CLI				; disable interrupts

	MOV	[DI],AX			; store address
	MOV	[DI+2],BX		; store segment

	STI				; enable interrupts

	POP	DS
	RET

KB_RST	ENDP

	


; <<<<<<<<<< Keyboard Interrupt Handler >>>>>>>>>>
;
; Description:
;
;       When a key is pressed on the keyboard, an interrupt will be
;       generated to this routine. This routine will then handle
;       the generation of an interrupt acknowledge and store the
;       character typed in KEYCHR.


KEYINT	PROC	FAR

	STI				; allow further interrupts

	PUSH	AX
	PUSH	CX
	PUSH	SI
	PUSH	DS			; save registers

	CLD				; forward direction

	MOV	AX,CS:DATASEG
	MOV	DS,AX			; set DS to data seg

	IN	AL,KB_DATA		; read in the character
	PUSH	AX			; save character
	IN	AL,KB_CNTL		; read control port
	MOV	AH,AL			; save value
	OR	AL,80H			; reset bit for keyboard
	OUT	KB_CNTL,AL
	XCHG	AH,AL			; get back original control
	OUT	KB_CNTL,AL		; keyboard has been reset

	POP	AX			; get back character pressed
	MOV	KEYCHR,AL		; store character

	CALL	KB_DCD			; keyboard decode routine

	CLI				; turn off interrupts
	MOV	AL,20H
	OUT	20H,AL			; send end of interrupt to 8259

	POP	DS
	POP	SI
	POP	CX
	POP	AX			; restore registers

	IRET				; return from interrupt

KEYINT	ENDP




; ========== Keyboard Decode Routine ==========
;
; On Entry:
;
;       KEYCHR = key that has been pressed
;
; Description:
;
;       This routine will decode the key pressed and if valid, will set one
;       of the following variables:
;
;       PDIR	- player direction -  0	= stop
;                                     1	= left
;                                     2	= right
;
;       FIRE	- fire flag	   -  0	= fire key not pressed
;                                     1	= fire key pressed
;
;	ENDGAME	- end game         -  0	= play game
;                                     1 = end game


KB_DCD	PROC	NEAR

	MOV	SI,OFFSET DGROUP:KB_TABLE	; point to table
	MOV	AL,KEYCHR			; get key pressed
	MOV	CX,NUM_CHR			; load number of characters

; ----- loop until character found in table or end of table

KD1:	CMP	AL,[SI]			; compare character to table
	JE	KD2			; jump if the same
	ADD	SI,3			; point to next character
	LOOP	KD1			; loop if not all characters checked

	RET				; character doesn't match table entrys

; ----- jump to the address stored in the table

KD2:	INC	SI			; point to address
	JMP	WORD PTR [SI]		; jump to address pointed to by SI

KB_DCD	ENDP




; ========== Key Press Handler ==========
;
; Description:
;
;       The routines in this procedure are called when a valid key has been
;       pressed and the appropiate action will take place. Also, when
;       control-left arrow or control-right arrow has been pressed, the
;       screen will shift left or right if the display is the color card.
;
;       The following PUBLIC variables are used:
;
;	PDIR	-     0	= stop
;		      1	= left   ( left arrow )
;		      2 = right  ( right arrow )
;
;	LDIR	-     1	= left   ( left arrow has been pressed )
;		      2 = right  ( right arrow has been pressed )
;
;	FIRE	- TRUE	= key is down  ( F1 )
;		  FALSE	= key is up
;
;	FIRES	- TRUE  = key has been pressed  ( F1 ) - program must reset variable
;
;	SOUND	- TRUE	= enable sound  ( F10 )
;		  FALSE = disable sound
;
;	ENDGAME	- TRUE	= Esc has been pressed



KEY_PRS	PROC	NEAR

; ----- left arrow key down

LEFT_DOWN:

	MOV	L_KEY,TRUE		; set left key flag
	MOV	KEYDIR,DIRL		; Show present key is left
	MOV	LKDIR,DIRL		; Show that the last key pressed was left
	MOV	PDIR,LEFT		; set direction variable to left
	MOV	LDIR,LEFT		; set last direction variable to left
	CALL	SCREEN_ADJ		; call screen adjust program
	RET

; ----- left arrow key up

LEFT_UP:

	MOV	L_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRL
	JNZ	KSP1
	CALL	KEYCHK
KSP1:	CMP	PDIR,LEFT
	JE	KP1			; jump if left arrow key is down
	RET				; return in it is not

KP1:	CMP	R_KEY,TRUE
	JNE	KP2			; jump if right key not down
	MOV	PDIR,RIGHT		; else set PDIR to right
	RET

KP2:	MOV	PDIR,STOP		; indicate key has been released
	RET				; by setting PDIR to stop

; ----- right arrow key down

RIGHT_DOWN:

	MOV	R_KEY,TRUE		; set left key flag
	MOV	KEYDIR,DIRR		; Show present key is right
	MOV	LKDIR,DIRR		; Show that the last key pressed was right
	MOV	PDIR,RIGHT		; set direction variable to right
	MOV	LDIR,RIGHT		; set last direction variable to right
	CALL	SCREEN_ADJ		; call screen adjust routine
	RET

; ----- right arrow key up

RIGHT_UP:

	MOV	R_KEY,FALSE		; set right key flag
	CMP	KEYDIR,DIRR
	JNZ	KSP2
	CALL	KEYCHK
KSP2:	CMP	PDIR,RIGHT
	JE	KP3			; jump if right arrow key is down
	RET

KP3:	CMP	L_KEY,TRUE
	JNE	KP4			; jump if left key not down
	MOV	PDIR,LEFT		; else set PDIR to left
	RET

KP4:	MOV	PDIR,STOP		; set direction to stop
	RET



; ----- up arrow key down

UP_DOWN:

	MOV	U_KEY,TRUE		; set left key flag
	MOV	KEYDIR,DIRU		; Show present key is left
	MOV	LKDIR,DIRU		; Show that the last key pressed was left
	RET

; ----- up arrow key up

UP_UP:

	MOV	U_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRU
	JNZ	KSP3
	CALL	KEYCHK
KSP3:	RET				; return in it is not


; ----- down arrow key down

DOWN_DOWN:

	MOV	D_KEY,TRUE		; set key flag
	MOV	KEYDIR,DIRD		; Show present key
	MOV	LKDIR,DIRD		; Show that the last key pressed 
	RET

; ----- down arrow key up

DOWN_UP:

	MOV	D_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRD
	JNZ	KSP4
	CALL	KEYCHK
KSP4:	RET				; return in it is not


; ----- up-right arrow key down

UPR_DOWN:

	MOV	UR_KEY,TRUE		; set key flag
	MOV	KEYDIR,DIRUR		; Show present key
	MOV	LKDIR,DIRUR		; Show that the last key pressed 
	RET

; ----- up-right arrow key up

UPR_UP:

	MOV	UR_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRUR
	JNZ	KSP5
	CALL	KEYCHK
KSP5:	RET				; return in it is not


; ----- up-left arrow key down

UPL_DOWN:

	MOV	UL_KEY,TRUE		; set key flag
	MOV	KEYDIR,DIRUL		; Show present key
	MOV	LKDIR,DIRUL		; Show that the last key pressed 
	RET

; ----- up-left arrow key up

UPL_UP:

	MOV	UL_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRUL
	JNZ	KSP6
	CALL	KEYCHK
KSP6:	RET				; return in it is not


; ----- down-right arrow key down

DOWNR_DOWN:

	MOV	DR_KEY,TRUE		; set key flag
	MOV	KEYDIR,DIRDR		; Show present key
	MOV	LKDIR,DIRDR		; Show that the last key pressed 
	RET

; ----- down-right arrow key up

DOWNR_UP:

	MOV	DR_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRDR
	JNZ	KSP7
	CALL	KEYCHK
KSP7:	RET				; return in it is not


; ----- down-left arrow key down

DOWNL_DOWN:

	MOV	DL_KEY,TRUE		; set key flag
	MOV	KEYDIR,DIRDL		; Show present key
	MOV	LKDIR,DIRDL		; Show that the last key pressed 
	RET

; ----- down-left arrow key up

DOWNL_UP:

	MOV	DL_KEY,FALSE		; set left key flag
	CMP	KEYDIR,DIRDL
	JNZ	KSP8
	CALL	KEYCHK
KSP8:	RET				; return in it is not




; ----- fire key down

FIRE_DOWN:

	MOV	FIRES,TRUE		; set fire key flag
	MOV	FIRE,TRUE		; set fire variable to true
	RET

; ----- fire key up

FIRE_UP:

	MOV	FIRE,FALSE		; set fire variable to false
	RET

; ----- control key down

RESET1_DOWN:

	MOV	RST1,TRUE		; indicate control key is down
	JMP	CHECK_END		; check for end of game

; ----- control key up

RESET1_UP:

	MOV	RST1,FALSE		; indicate control key up
	RET

; ----- alternale key down

RESET2_DOWN:

	MOV	RST2,TRUE		; indicate alternate key down
	JMP	CHECK_END		; check for end of game

; ----- alternate key up

RESET2_UP:

	MOV	RST2,FALSE		; indicate alternate key up
	RET

; ----- delete key down

RESET3_DOWN:

	MOV	RST3,TRUE		; indicate delete key down
	JMP	CHECK_END		; check for end of game

; ----- delete key up

RESET3_UP:

	MOV	RST3,FALSE		; indicate delete key up
	RET

; ----- F9 down

SOUND_DOWN:

	CMP	SND_UP,TRUE
	JE	SD1			; jump if F9 was not down before
	RET				; else return

SD1:	XOR	SOUND,TRUE		; change the state of SOUND
	MOV	SND_UP,FALSE		; indicate F9 was down
	RET

; ----- F10 up

SOUND_UP:

	MOV	SND_UP,TRUE		; indicate F10 was up
	RET
	
; ----- Esc down

EXIT_DOWN:

	MOV	ENDGAME,TRUE		; indicate Esc was down
	RET





KEY_PRS	ENDP


; ======= key down check ======
;
; Discription:
;
;	This routine checks if any of the 8 direction keys are still down
;	if a match is found, it is treated as a new key press
;

KEYCHK	PROC	NEAR

	MOV	KEYDIR,99
	CMP	U_KEY,0
	JZ	CHK1		; Key is not pressed
	MOV	KEYDIR,DIRU
	MOV	LKDIR,DIRU
CHK1:	CMP	D_KEY,0
	JZ	CHK2		; Key is not pressed
	MOV	KEYDIR,DIRD
	MOV	LKDIR,DIRD
CHK2:	CMP	L_KEY,0
	JZ	CHK3		; Key is not pressed
	MOV	KEYDIR,DIRL
	MOV	LKDIR,DIRL
CHK3:	CMP	R_KEY,0
	JZ	CHK4		; Key is not pressed
	MOV	KEYDIR,DIRR
	MOV	LKDIR,DIRR
CHK4:	CMP	UR_KEY,0
	JZ	CHK5		; Key is not pressed
	MOV	KEYDIR,DIRUR
	MOV	LKDIR,DIRUR
CHK5:	CMP	UL_KEY,0
	JZ	CHK6		; Key is not pressed
	MOV	KEYDIR,DIRUL
	MOV	LKDIR,DIRUL
CHK6:	CMP	DR_KEY,0
	JZ	CHK7		; Key is not pressed
	MOV	KEYDIR,DIRDR
	MOV	LKDIR,DIRDR
CHK7:	CMP	DL_KEY,0
	JZ	CHK8		; Key is not pressed
	MOV	KEYDIR,DIRDL
	MOV	LKDIR,DIRDL
CHK8:	RET

KEYCHK	ENDP

; ===== Game End Check =====
;
; Description:
;
;       This routine will check to see if the control-alternate-delete
;       keys are down. If they are, a reboot will occour.


CHECK_END	PROC	NEAR

	CMP	RST1,TRUE
	JE	BC1			; jump if control key is down
	RET				; else return

BC1:	CMP	RST2,TRUE
	JE	BC2			; jump if alternate key is down
	RET				; else return

BC2:	CMP	RST3,TRUE
	JE	BC3			; jump if delete key is down
	RET				; else return

BC3:	MOV	AX,40H
	MOV	DS,AX			; set data seg to rom data segment

	ASSUME	DS:ROMDATA

	MOV	RESET_FLAG,1234H	; signal keyboard reset
	JMP	FAR PTR REBOOT		; jump to warm boot routine

	ASSUME	DS:DGROUP


CHECK_END	ENDP



; ===== Screen Adjust Routine =====
;
; Description:
;
;       This routine will determine if a control-left arrow or
;       control-right arrow is pressed. If they are and the display
;       is the color card, then the horizontal sync position will be
;       altered to cause the screen to move left or right.


SCREEN_ADJ	PROC	NEAR

	CMP	RST1,TRUE
	JE	SA2			; jump if control key is down
	RET				; else return

SA2:	CMP	L_KEY,TRUE
	JNE	SA3			; jump if left arrow key is not down
	INC	SYNC_POS		; increment the sync position
	CMP	SYNC_POS,SYNC_NORM+5	; check for limit
	JNZ	SBB1
	MOV	SYNC_POS,SYNC_NORM+4	; reset sync position
SBB1:	PUSH	DX
	MOV	DX,3D4H			; address of register select port
	MOV	AL,2			; horizontal sync position register
	OUT	DX,AL			; send to CRT controller ( 6845 )
	MOV	AL,SYNC_POS		; get sync position
	INC	DX			; point to data register of 6845
	OUT	DX,AL			; send to 6845
	MOV	CRTINIT+2,AL		; Set initialization table for next time
	POP	DX
	RET

SA3:	CMP	R_KEY,TRUE
	JNE	SA4			; jump if right arrow key is not down
	DEC	SYNC_POS		; decrement the sync position
	CMP	SYNC_POS,SYNC_NORM-5	; check for limit
	JNZ	SBB2
	MOV	SYNC_POS,SYNC_NORM-4	; reset sync position
SBB2:	PUSH	DX
	MOV	DX,3D4H			; address of register select port
	MOV	AL,2			; horizontal sync position register
	OUT	DX,AL			; send to CRT controller ( 6845 )
	MOV	AL,SYNC_POS		; get sync position
	INC	DX			; point to data register of 6845
	OUT	DX,AL			; send to 6845
	MOV	CRTINIT+2,AL		; Set initialization table for next time
	POP	DX
	RET

SA4:	RET

SCREEN_ADJ	ENDP




; ========== Peek Routine ==========
;
; Command Format:
;
;       n=peek(addr);
;	int  addr;
;
; Description:
;
;       The value at location addr and segment defined
;       by defseg will returned.


PEEK	PROC	NEAR

	PUSH	BP
	MOV	BP,SP
	PUSH	DS

	MOV	SI,[BP]+4		; put address in BX
	MOV	CX,DEFSEG		; get data segment
	MOV	DS,CX			; and put in DS

	MOV	AL,[SI]			; get value into AX
	MOV	AH,0			; clear high byte

	POP	DS
	POP	BP
	RET

PEEK	ENDP



; ========== Poke Routine ==========
;
; commamd format:
;
;       poke(addr,byte);
;       int	addr;
;	char	byte;
;
; description:
;
;       The value byte will be placed in location addr
;       and segment defseg.


POKE	PROC	NEAR

	PUSH	BP
	MOV	BP,SP
	PUSH	DS

	MOV	DI,[BP]+4		; get address
	MOV	BX,[BP]+6		; get data byte
	MOV	CX,DEFSEG
	MOV	DS,CX			; set DS to proper segment

	MOV	[DI],BL			; write byte

	POP	DS
	POP	BP
	RET

POKE	ENDP



PROG	ENDS
	END
